/** file ifs.cc
 * this program is for testing various ifs systems.
 * reads a file with urls as produced by ripper --caturl as input
 */
/* 
 * Copyright (C) 2002 Laird Breyer
 *  
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 * Author:   Laird Breyer <laird@lbreyer.com>
 */

#include <plotter.h>
#include <math.h>
#include <ctype.h>
#include <iostream>
#include <gsl/gsl_randist.h>

typedef struct {
  float coordx;
  float coordy;
} Coords;


int main(int argc, char** argv) {

  Coords *urls = new Coords[1000000];
  XPlotter plotter;

    gsl_rng* r;
    r = gsl_rng_alloc(gsl_rng_taus);

    double xuniforms[2048], yuniforms[2048];
    for(int z = 0; z < 2048; z++) {
      xuniforms[z] = gsl_rng_uniform(r);
      yuniforms[z] = gsl_rng_uniform(r);
    }


  double minx, miny, maxx, maxy;
  char sbuf[4096];
  int k = 0;
  while(!cin.eof() && (k < 350000)) {
    cin >> sbuf;
    if( k % 10000 == 0 ) { cout << "*"; cout.flush(); }
    urls[k].coordx = 0.0;
    urls[k].coordy = 0.0;



    double cc, sc, nx, ny;
    char *p = strstr(sbuf,"//");
    int s = 0;
    for(p = strstr(p,"."); *(++p); s++) 
      {
	unsigned char c = *p;
	double cval = (double)c;

	nx = urls[k].coordx + log(cval);
	ny = urls[k].coordy;

	double alpha = 6.28 * cval / 128;
	cc = cos(alpha);
	sc = sin(alpha);

	urls[k].coordx = cc * nx - sc * ny;
	urls[k].coordy = sc * nx + cc * ny;
      }	


    minx = (minx < urls[k].coordx) ? minx : urls[k].coordx;
    miny = (miny < urls[k].coordy) ? miny : urls[k].coordy;
    maxx = (maxx > urls[k].coordx) ? maxx : urls[k].coordx;
    maxy = (maxy > urls[k].coordy) ? maxy : urls[k].coordy;
    k++;
  }
  cout << "\nNow opening plotter with limits " << minx << ", " << miny
       << "  " << maxx << ", " << maxy << endl;
  plotter.openpl();
  plotter.fspace(minx, miny, maxx, maxy);
  plotter.erase();
  plotter.pencolorname("black");
  for(int i = 0; i < k; i++) {
    plotter.point(urls[i].coordx, urls[i].coordy);
  }
  plotter.closepl();
}





// CHAOS
//       double alpha, cc, sc, nx, ny;
//       {
// 	double lambda = 0.9995;
// 	unsigned char c = p[0];
// 	if( c == '/' ) {
// 	  nx = urls[k].coordx + lambda*10.0;
// 	  ny = urls[k].coordy;
// 	  alpha = -2.1;
// 	} else if (c == '.' ) {
// 	  nx = urls[k].coordx + lambda*10.0;
// 	  ny = urls[k].coordy;
// 	  alpha = 3.15;
// 	} else if (isalpha(c)) {
// 	  //	  alpha = 0.0005 * ( 1 + (tolower(c)-61)/26.0); //0; 0.01 * (c - 65);
// 	  alpha = 0.0005;
// 	  // nx = urls[k].coordx; // + 0.5;
// 	} else {
// 	  alpha = 0.0005;
// 	}
// 	cc = cos(alpha);
// 	sc = sin(alpha);
// 	urls[k].coordx = lambda * (cc * nx + sc * ny);
// 	urls[k].coordy = lambda * (-sc * nx + cc * ny);

//       }
