/** @file linkgraph.h */
/* 
 * Copyright (C) 2002 Laird Breyer
 *  
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 * Author:   Laird Breyer <laird@lbreyer.com>
 */

#ifndef _GRAPHER_H_
#define _GRAPHER_H_
//#include "document.h"
#include "webnode.h"
#include <slist>

/// A singly linked list of WebNodePtr objects
typedef slist<WebNode*> WebNodeList;

/// Encapsulates the web link graph as a WebNodeList of WebNode objects.
/**
 * This class offers several enhancements over a WebNodeList,
 * including the ability to print a description of 
 * the graph to a stream and the ability to construct tagged
 * fromsets.
 *
 * The node id's are in decreasing order, since nodes are pushed at the front
 * by GraphBuilder(). This ordering should not be changed, since other parts
 * (eg Talker() ) depend on it.
 */
class WebLinkGraph: public WebNodeList {
 public:

  WebLinkGraph();

  void PrintWebGraph(ostream& o);

  void ClearTags();
  void BuildFromSets(int k);
  void MeasureFromSets();
  /// returns fromsetsizes[] - needed by some WebSampler objects.
  const uint32 *GetFromSetSizes() {
    return fromsetsize;
  }
  void StatisticsFromSets(ostream& o);
  
 private:
  uint32 fromsetsize[TAG_NUMBER_OF_BITS];
};

#endif
