/** @file pvminterface.h */
/* 
 * Copyright (C) 2002 Laird Breyer
 *  
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 * Author:   Laird Breyer <laird@lbreyer.com>
 */

#ifndef _PVM_INTERFACE_H
#define _PVM_INTERFACE_H

#ifdef DISTRIBUTED_COMPUTING
// don't inclued anything above this line
#include <pvm3.h>
#endif

#include "basictypes.h"
#include <stdexcept>

#define PVMINTERFACE_NAMELEN 20
#define PVMINTERFACE_SENDINFO 1
#define PVMINTERFACE_SENDCOMMAND 2
#define PVMINTERFACE_SENDLEAFUPD 3

/// Contains vital task information.
struct PVMTaskInfo {
  int tid;
  uint32 start_id;
  uint32 stop_id;
  char name[PVMINTERFACE_NAMELEN+1];
};

/// Handles communication between Talker and PVM
class PVMInterface {
 public:
  PVMInterface(const char* name, 
	       bool ismaster, int ntasks, 
	       int stardid, int stopid) throw (runtime_error);
  ~PVMInterface();

  void BroadcastCommand(const char* commandline) throw (runtime_error);
  void SendCommand(int taskno, const char* commandline) throw (runtime_error);
  void GetCommandWithTimeout(char* commandline) throw (runtime_error);

  bool MessagePending();
  int LastMessageOriginator();

  bool AllStandby() throw (runtime_error);
  void LeaveStandbyGroup();
  void JoinStandbyGroup();

  int FindTask(uint32 id);
  const char* Name(int k);
  int NumberOfTasks() 
    { return num_tasks; }
  int NumberOfOtherTasks() 
    { return num_tasks - 1; }
  uint32 StartID(int k) 
    { assert(k < num_tasks - 1); return tasklist[k].start_id; }
  uint32 StopID(int k) 
    { assert(k < num_tasks - 1); return tasklist[k].stop_id; }

  bool IsMaster() 
    { return is_master; }
  bool IsSlave() 
    { return !(is_master || (num_tasks > 0)); }

  void GetLeafCounts(unsigned int* countsarray, uint32* size) throw (runtime_error);
  void SendLeafCounts(int tid, unsigned int* countsarray, uint32 size) throw (runtime_error);  

 private:
  PVMTaskInfo myinfo;
  bool is_master;
  int num_tasks;
  
  PVMTaskInfo *tasklist;

  int last_message_tid;
  int last_message_size;
  int last_message_msgtag;
};

#endif
